# coding: utf-8

"""
   Python SDK for OpenFGA

   API version: 0.1
   Website: https://openfga.dev
   Documentation: https://openfga.dev/docs
   Support: https://discord.gg/8naAwJfWN6
   License: [Apache-2.0](https://github.com/openfga/python-sdk/blob/main/LICENSE)

   NOTE: This file was auto generated by OpenAPI Generator (https://openapi-generator.tech). DO NOT EDIT.
"""

import re


def is_well_formed_ulid_string(ulid):
    regex = re.compile("^[0-7][0-9A-HJKMNP-TV-Z]{25}$")
    if not isinstance(ulid, str):
        return False
    match = regex.match(ulid)
    if match is None:
        return False
    return True
