# coding: utf-8
"""
   Python SDK for OpenFGA

   API version: 0.1
   Website: https://openfga.dev
   Documentation: https://openfga.dev/docs
   Support: https://discord.gg/8naAwJfWN6
   License: [Apache-2.0](https://github.com/openfga/python-sdk/blob/main/LICENSE)

   NOTE: This file was auto generated by OpenAPI Generator (https://openapi-generator.tech). DO NOT EDIT.
"""

from openfga_sdk.client.models.write_single_response import ClientWriteSingleResponse

from typing import List


class ClientWriteResponse():
    """
    ClientWriteResponse returns the set of responses and their statuses
    """

    def __init__(self, writes: List[ClientWriteSingleResponse], deletes: List[ClientWriteSingleResponse]):
        self._writes = writes
        self._deletes = deletes

    @property
    def writes(self):
        """
        Return the writes response
        """
        return self._writes

    @property
    def deletes(self):
        """
        Return the delete response
        """
        return self._deletes
